/*
 * scientific visualization api for direct framebuffer
 *
 * author: hackbard@hackdaworld.dyndns.org
 *
 */

#include <stdio.h>
#include "dfbapi.h"

/* two dimensional lattice */

int d2_lattice_init(int *argc,char **argv,d2_lattice *d2_l,int x,int y)
{
 DFBSurfaceDescription surface_dsc;
 DFBFontDescription font_dsc;
 
 d2_l->max_x=x;
 d2_l->max_y=y;

 DirectFBInit(argc,&argv);
 DirectFBCreate(&(d2_l->dfb));
 d2_l->dfb->SetCooperativeLevel(d2_l->dfb,DFSCL_FULLSCREEN);

 surface_dsc.flags=DSDESC_CAPS;
 surface_dsc.caps=DSCAPS_PRIMARY|DSCAPS_FLIPPING;
 d2_l->dfb->CreateSurface(d2_l->dfb,&surface_dsc,&(d2_l->p_surface));
 d2_l->p_surface->GetSize(d2_l->p_surface,&(d2_l->s_width),&(d2_l->s_height));
 
 font_dsc.flags=DFDESC_HEIGHT;
 font_dsc.height=d2_l->max_y*((d2_l->s_height)/(d2_l->max_y))/40; /* 40 ? */
 d2_l->font_h=font_dsc.height;
 d2_l->dfb->CreateFont(d2_l->dfb,FONT,&font_dsc,&(d2_l->font));
 d2_l->p_surface->SetFont(d2_l->p_surface,d2_l->font);

 d2_l->fakt_y=(d2_l->s_height)/(d2_l->max_y+(2*Y_GAP));
 d2_l->fakt_x=(d2_l->s_height)/(d2_l->max_x+X_GAP); /* bullshit, i can't imagine atm */
 d2_l->info_x=d2_l->fakt_x*d2_l->max_x+2*X_GAP;
 d2_l->info_y=Y_GAP;
 d2_l->info_w=d2_l->s_width-d2_l->info_x-2*X_GAP;
 d2_l->info_h=d2_l->max_y*d2_l->fakt_y;

 if((d2_l->s_height<(d2_l->max_y+2*Y_GAP)) || (d2_l->s_height<(d2_l->max_x+X_GAP)))
 {
  puts("resolution too low!");
  return -1;
 } else return 1;
}

int d2_lattice_release(d2_lattice *d2_l)
{
 d2_l->font->Release(d2_l->font);
 d2_l->p_surface->Release(d2_l->p_surface);
 d2_l->dfb->Release(d2_l->dfb);

 return 1;
}

int d2_lattice_get_color(unsigned char *status,unsigned char *r,unsigned char *g,unsigned char *b)
{
 if((*status)&RED)
 {
  *r=0xff;
  *g=0;
  *b=0;
 } else
 {
  *r=0;
  *g=0;
  *b=0xff;
 }
 
 return 1;
}

int d2_lattice_draw(d2_lattice *d2_l,int x,int y,int arg_c,char **arg_v)
{
 int x_c,y_c;
 int i;
 unsigned char r,g,b,a;

 a=0xff; /* no alpha blending */
 
 for(x_c=0;x_c<d2_l->max_x;x_c++)
 {
  for(y_c=0;y_c<d2_l->max_y;y_c++)
  {
   d2_lattice_get_color((*d2_l).status+x_c+y_c*d2_l->max_x,&r,&g,&b);
   if(x_c==x && y_c==y)
   {
    r=0xff;
    g=0xff;
    b=0;
   }
   d2_l->p_surface->SetColor(d2_l->p_surface,r,g,b,a);
   d2_l->p_surface->DrawRectangle(d2_l->p_surface,
                                  x_c*d2_l->fakt_x+X_GAP,
                                  y_c*d2_l->fakt_y+Y_GAP,
                                  d2_l->fakt_x,d2_l->fakt_y);
  }
 }
 r=0xff;
 g=0xff;
 b=0;
 /* clear info box */
 d2_l->p_surface->SetColor(d2_l->p_surface,0,0,0,0);
 d2_l->p_surface->FillRectangle(d2_l->p_surface,
                                d2_l->info_x,d2_l->info_y,
                                d2_l->info_w,d2_l->info_h);
 d2_l->p_surface->SetColor(d2_l->p_surface,r,g,b,a);
 d2_l->p_surface->DrawRectangle(d2_l->p_surface,
                                d2_l->info_x,d2_l->info_y,
                                d2_l->info_w,d2_l->info_h);
 d2_l->p_surface->SetColor(d2_l->p_surface,0x80,0x80,0xff,0xff);
 for(i=1;i<=arg_c;i++)
 {
  d2_l->p_surface->DrawString(d2_l->p_surface,arg_v[i],-1,
                              d2_l->info_x+d2_l->fakt_x,
                              d2_l->info_y+d2_l->fakt_y+d2_l->font_h+(i-1)*2*d2_l->font_h,
                              DSTF_LEFT);
 }
 /* now we flip all to surface */
 d2_l->p_surface->Flip(d2_l->p_surface,NULL,0);
 
 return 1;
}
  
